{
   StringUtils, string manipulation utilities
   Copyright (C) 2010. Dejan Boras

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   Started On:    06.08.2010.
   Last Update:   12.05.2011.
}

{$MODE OBJFPC}{$H+}
UNIT StringUtils;

INTERFACE

CONST
   strWhitespace       = [' ', #9];

TYPE
   TShortStringArray = array of shortstring;
   TpShortStringArray = array of pshortstring;
   TpCharDynArray = array of pchar;
   TAnsiStringArray = array of ansistring;

{STRING TO NUMERICAL VALUE}
{make a string out of a value}
function dStr(value: byte): string; inline;
function dStr(value: word): string; inline;
function dStr(value: longword): string; inline;
function dStr(value: uint64): string; inline;
function dStr(value: shortint): string; inline;
function dStr(value: smallint): string; inline;
function dStr(value: longint): string; inline;
function dStr(value: int64): string; inline;
function dStr(value: single; dec: longint): string; inline;
function dStr(value: single): string; inline;
function dStr(value: double; dec: longint): string; inline;
function dStr(value: double): string; inline;
function dStr(value: boolean): string; inline;

{compares 2 strings}
function dCompareStrings(const str1, str2: string): longint;
{compares 2 strings up to the length of the smallest string}
function dCompareStringz(const str1, str2: string): longint;

{WHITESPACE STRIPPING}
procedure dStripLeadingWhitespace(var st: string);
procedure dStripTrailingWhitespace(var st: string);
procedure dStripWhitespace(var st: string);

{completely remove whitespace from the string}
procedure dEliminateWhiteSpace(var st: string);
{remove a character from the string}
procedure dRemoveChar(var st: string; c: char);

{FILE NAME ROUTINES}
{extract the file name from a string}
function dExtractFileName(const st: string): string;
{extracts the file name without the extension}
function dExtractFileNameNoExt(const st: string): string;
{extracts everything without the extension}
function dExtractAllNoExt(const st: string): string;
{extract the file directory from a string}
function dExtractFileDir(const st: string): string;
{extract the file extension from a string}
function dExtractFileExt(const st: string): string;
{extract the file path from a string}
function dExtractFilePath(const st: string): string;
{extract the file drive from a string, good only for Win32}
function dExtractFileDrive(const st: string): string;
{replace directory separators with the one used on the current platform}
procedure dReplaceDirSeparators(var st: string);

{ SUB STRINGS }

{returns a substring from the string st starting at index position idx to the end}
function dStringFrom(const st: string; idx: longint): string;
{returns a substring from the string st starting at index 1 to the position idx}
function dStringTo(const st: string; idx: longint): string;

{copy until whitespace and delete from string}
function dCopy2Del(var s: string): string;
{copy until the specified character is found and delete from string}
function dCopy2Del(var s: string; c: char): string;

{add the specified leading character to make the string have length n}
procedure dStringAddPaddingLeading(var s: string; c: char; n: longint);
{add the specified trailing character to make the string have length n}
procedure dStringAddPaddingTrailing(var s: string; c: char; n: longint);

{ SHORTSTRINGS }
{creates a pshortstring from a regular shortstring or a pchar}
function dMakepShortString(const s: string): pShortString;
function dMakepShortString(pcs: pChar): pShortString;

{converts a shortstring into pchar; the memory allocated by this function
must be freed afterwards}
function dShortStringToPChar(const s: ShortString): pChar;
{'converts' a pchar to a shortstring}
function dPCharToShortString(pcs: pChar): ShortString;

{convert a string to a pchar with memory allocation}
function dStringTopChar(const s: string): pChar;

IMPLEMENTATION

CONST
   DirectorySeparators = ['/', '\'];

{STRING TO NUMERICAL VALUE}

function dStr(value: byte): string; inline;
begin
   str(value, result);
end;

function dStr(value: word): string; inline;
begin
   str(value, result);
end;

function dStr(value: longword): string; inline;
begin
   str(value, result);
end;

function dStr(value: uint64): string; inline;
begin
   str(value, result);
end;

function dStr(value: shortint): string; inline;
begin
   str(value, result);
end;

function dStr(value: smallint): string; inline;
begin
   str(value, result);
end;

function dStr(value: longint): string; inline;
begin
   str(value, result);
end;

function dStr(value: int64): string; inline;
begin
   str(value, result);
end;

function dStr(value: single; dec: longint): string; inline;
begin
   str(value:0:dec, result);
end;

function dStr(value: single): string;
begin
   str(value, result);
end;

function dStr(value: double; dec: longint): string; inline;
begin
   str(value:0:dec, result);
end;

function dStr(value: double): string;
begin
   str(value, result);
end;


function dStr(value: boolean): string; inline;
begin
   case value of
      true:  result := 'true';
      false: result := 'false';
   end;
end;

function dCompareStrings(const str1, str2: string): longint;
var
   i, l1, l2: longint;

begin
   l1 := Length(str1);
   l2 := Length(str2);
   if(l1 = l2) then begin
      if(l1 <> 0) or (l2 <> 0) then begin
         for i := 1 to l1 do begin
            if(str1[i] < str2[i]) then exit(-i)
            else if(str1[i] > str2[i]) then exit(i);
         end;
         result := 0;
      end else result := 0;
   end else result := -1;
end;

function dCompareStringz(const str1, str2: string): longint;
var
   i, l1, l2, sl: longint;

begin
   l1 := Length(str1);
   l2 := Length(str2);
   if(l1 = l2) then begin
      if(l1 <> 0) or (l2 <> 0) then begin
         if(l1 > l2) then sl := l1 else sl := l2;
         for i := 1 to sl do begin
            if(str1[i] < str2[i]) then exit(-i)
            else if(str1[i] > str2[i]) then exit(i);
         end;
         result := 0;
      end else result := 0;
   end else result := -1;
end;

{WHITESPACE STRIPPING}
procedure dStripLeadingWhitespace(var st: string);
var
   i, l, strippos: longint;

begin
   l := Length(st);
   if(l > 0) then begin
      {strip leading whitespace}
      {find whitespace}
      strippos := 0;
      for i := 1 to l do begin
         if(st[i] in strWhitespace) then inc(strippos)
         else break;
      end;

      {delete whitespace from the string}
      if(strippos <> 0) then begin
         delete(st, 1, strippos);
      end;
   end;
end;

procedure dStripTrailingWhitespace(var st: string);
var
   i, l, strippos: longint;

begin
   l := length(st);
   if(l > 0) then begin
      {strip trailing whitespace}
      {find whitespace}
      strippos := 0;
      for i := l downto 1 do begin
         if(st[i] in strWhitespace) then inc(strippos)
         else break;
      end;

      {delete whitespace from the string}
      if(strippos > 0) then begin
         delete(st, l-(strippos-1), strippos);
      end;
   end;
end;

procedure dStripWhitespace(var st: string);
begin
   dStripLeadingWhitespace(st);
   dStripTrailingWhitespace(st);
end;

procedure dEliminateWhiteSpace(var st: string);
var
   i, l, newlen, count: longint;

begin
   l := Length(st);
   if(l > 0) then begin
      newlen := l; count := 0;
      for i := 1 to l do begin
         if(st[i] in strWhiteSpace) then dec(newlen)
         else begin
            inc(count);
            st[count] := st[i]
         end;
      end;

      SetLength(st, newlen);
   end;
end;

procedure dRemoveChar(var st: string; c: char);
var
   i, l, newlen, count: longint;

begin
   l := Length(st);
   if(l > 0) then begin
      newlen := l; count := 0;
      for i := 1 to l do begin
         if(st[i] = c) then dec(newlen)
         else begin
            inc(count);
            st[count] := st[i]
         end;
      end;

      SetLength(st, newlen);
   end;
end;

{FILE NAME ROUTINES}
function dExtractFileName(const st: string): string;
var
   i, l: longint;

begin
   l := Length(st);
   if(l > 0) then begin
      i := l;
      while (i > 0) and (not(st[i] in DirectorySeparators)) do
         dec(i);

      if(i+1 < l) then
         result := copy(st, i+1, 255)
      else result := '';
   end else result := '';
end;

function dExtractFileNameNoExt(const st: string): string;
var
   i, l, e, dotpos: longint;

begin
   l := Length(st);
   if(l > 0) then begin
      i := l;
      while(i >= 1) and (not (st[i] in DirectorySeparators)) do
         dec(i);

      if(i <= l) then begin
         {now that we've found where the beginning of the filename is,
         we need to find where the end is(without the extension)}
         dotpos := pos(ExtensionSeparator, st);
         if(dotpos > 0) then e := dotpos else e := l;

         {now just copy the filename}
         result := copy(st, i+1, e-i-1);
      end else result := st;
   end else result := st;
end;

function dExtractAllNoExt(const st: string): string;
var
   i, l: longint;

begin
   l := Length(st);
   if(l <> 0) then begin
      i := l;
      while(i > 1) and (st[i] <> ExtensionSeparator) do
         dec(i);

      if(i > 1) then
         result := copy(st, 1, i-1)
      else
        result := st;
   end else result := '';
end;

function dExtractFileDir(const st: string): string;
var
   i, l: longint;

begin
   l := Length(st);
   if(l > 0) then begin
      i := l;
      while (i >= 1) and (not (st[i] in DirectorySeparators)) do
         dec(i);

      if(i >= 2) then
         result := copy(st, 1, i-1)
      else
         result := '';
   end else result := '';
end;

function dExtractFileExt(const st: string): string;
var
   i, l: longint;

begin
   l := Length(st);
   if(l > 0) then begin
      i := l;
      while (i >= 1) and (st[i] <> ExtensionSeparator) do
         dec(i);

      if(i > 0) then begin
         {extract the extension and return it}
         result := copy(st, i, 255);
      end else result := '';
   end else result := '';
end;

function dExtractFilePath(const st: string): string;
var
   i, l: longint;

begin
   l := Length(st);
   if(l > 0) then begin
     i := l;
     while (i >= 1) and (not (st[i] in DirectorySeparators)) do
        dec(i);

      if(i >= 2) then
         result := copy(st, 1, i)
      else
         result := '';
   end else result := '';
end;

function dExtractFileDrive(const st: string): string;
var
   i, l: longint;

begin
   l := Length(st);
   if(l > 1) then begin
      if(st[2] = DriveSeparator) then
         result := copy(st, 1, 2)
      else if(st[1] in DirectorySeparators) and (st[2] in DirectorySeparators) then begin
         {skip the share}
         i := 2;
         while (i < l) and not (st[i+1] in DirectorySeparators) do inc(i);
         inc(i);

         while(i < l) and not (st[i+1] in DirectorySeparators) do inc(i);

         result := copy(st, 1, i);
      end;
   end else result := '';
end;

procedure dReplaceDirSeparators(var st: string);
var
   rds: char; {the directory separator which must be replaced}
   i: longint;

begin
   {decide which separator to replace}
   {$IFDEF WINDOWS}rds := '/';{$ENDIF}
   {$IFDEF LINUX}  rds := '\';{$ENDIF}
   {$IFDEF MACOS}  rds := '\';{$ENDIF}

   for i := 1 to Length(st) do begin
      if(st[i] = rds) then st[i] := DirectorySeparator;
   end;
end;

{ SUB-STRINGS }

function dStringFrom(const st: string; idx: longint): string;
var
   l: longint;

begin
   {check values}
   l := Length(st);
   if(l > 0) then begin
      if(idx <= l) then begin
         result := copy(st, idx, (l-idx)+1);
      end else result := '';
   end else result := '';
end;

function dStringTo(const st: string; idx: longint): string;
var
   l: longint;

begin
   {check values}
   l := Length(st);
   if(l > 0) then begin
      if(l < idx) then idx := l;

      {return required string}
      result := copy(st, 1, idx);
   end else result := '';
end;

function dCopy2Del(var s: string): string;
var
   i, slen: longint;

begin
   slen := length(s);
   if(slen > 0) then begin
      i := 1;
      while (i <= slen) and (not (s[i] in strWhiteSpace)) do inc(i);

      result := copy(s, 1, i-1);
      delete(s, 1, i);
   end else result := '';
end;

function dCopy2Del(var s: string; c: char): string;
var
   i, slen: longint;

begin
   slen := length(s);
   if(slen > 0) then begin
      i := 1;
      while (i <= slen) and (s[i] <> c) do inc(i);

      result := copy(s, 1, i-1);
      delete(s, 1, i);
   end else result := '';
end;

procedure dStringAddPaddingLeading(var s: string; c: char; n: longint);
var
   i, l, m: longint;

begin
   l := Length(s);
   if(n > l) then begin
      m := n-length(s);
      {set the new length}
      SetLength(s, n);
      {move characters}
      move(s[1], s[1+m], l);
      {add leading padding}
      for i := 1 to m do s[i] := c;
   end;
end;

procedure dStringAddPaddingTrailing(var s: string; c: char; n: longint);
var
   i, l, m: longint;

begin
   l := Length(s);
   if(n > l) then begin
      m := n-(n-l);
      {set the new length}
      SetLength(s, n);
      {add trailing padding}
      for i := m+1 to n do s[i] := c;
   end;
end;

{ SHORTSTRING }
function dMakepShortString(const s: string): pShortString;
var
   ps: pShortString = nil;

begin
   dMakepShortString := nil;

   GetMem(ps, Length(s)+1);
   if(ps <> nil) then begin
      ps^ := s;
      dMakepshortstring := ps;
   end;
end;

function dMakepShortString(pcs: pChar): pShortString;
var
   ss: pShortString = nil;
   len: longint = 0;
   pc: pChar;

begin
   if(pcs <> nil) then begin
      {figure out the string length}
      pc := pcs;
      repeat
         if(pc^ <> #0) then begin inc(pc); inc(len); end;
      until (pc^ = #0) or (len = 255);

      {get enough memory for the pShortString}
      GetMem(ss, len+1);
      if(ss <> nil) then begin
         {copy the string}
         if(len > 0) then move(pcs^, ss^[1], len);
         ss^[0] := char(len); {set string length}
      end;
   end;

   dMakepShortString := ss;{return result}
end;

function dShortStringToPChar(const s: ShortString): pChar;
var
   pcs: pChar = nil;
   stlen: longint;

begin
   stlen := Length(s);
   GetMem(pcs, stlen+1);
   if(pcs <> nil) then begin
      if(stlen > 0) then move(s[1], pointer(pcs)^, stlen);
      char(pointer(pcs+stlen)^) := #0;
   end;

   dShortStringToPChar := pcs;
end;

function dPCharToShortString(pcs: pChar): ShortString;
var
   ss: ShortString = '';
   len: longint = 0;
   pc: pChar = nil;

begin
   dPCharToShortString := '';

   {figure out the string length}
   pc := pcs;
   repeat
      if(pc^ <> #0) then begin
         inc(len); ss[len] := pc^; inc(pc);
      end;
   until (pc^ = #0) or (len = 255);

   {copy the string}
   ss[0] := char(len); {set string length}

   dPCharToShortString := ss;
end;

function dStringTopChar(const s: string): pChar;
var
   pcs: pChar = nil;
   stlen: longint;

begin
   stlen := Length(s);
   GetMem(pcs, stlen+1);
   if(pcs <> nil) then begin
      if(stlen > 0) then move(s[1], pointer(pcs)^, stlen);
      char(pointer(pcs+stlen)^) := #0;
   end;

   result := pcs;
end;


END.
