{
   uColors, color types, constants & routines
   Copyright (C) Dejan Boras 2009.

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   Started On:    16.11.2008.
   Last Update:   02.02.2011.
}

{$MODE OBJFPC}{$H+}
UNIT uColors;

INTERFACE

   USES StringUtils;

TYPE
   {COLOR TYPES}
   {Not all of these are used, but regardless of that they may prove usefull sometime.}
   {unsigned byte}
   PColor1ub = ^TColor1ub;    TColor1ub = byte;
   PColor2ub = ^TColor2ub;    TColor2ub = array[0..1] of byte;
   PColor3ub = ^TColor3ub;    TColor3ub = array[0..2] of byte;
   PColor4ub = ^TColor4ub;    TColor4ub = array[0..3] of byte;

   {byte}
   PColor1b = ^TColor1b;    TColor1b = shortint;
   PColor2b = ^TColor2b;    TColor2b = array[0..1] of shortint;
   PColor3b = ^TColor3b;    TColor3b = array[0..2] of shortint;
   PColor4b = ^TColor4b;    TColor4b = array[0..3] of shortint;
   
   {unsigned short}
   PColor1us = ^TColor1us;    TColor1us   = word;
   PColor2us = ^TColor2us;    TColor2us   = array[0..1] of word;
   PColor3us = ^TColor3us;    TColor3us   = array[0..2] of word;
   PColor4us = ^TColor4us;    TColor4us   = array[0..3] of word;

   {signed short}
   PColor1s  = ^TColor1s;     TColor1s    = smallint;
   PColor2s  = ^TColor2s;     TColor2s    = array[0..1] of smallint;
   PColor3s  = ^TColor3s;     TColor3s    = array[0..2] of smallint;
   PColor4s  = ^TColor4s;     TColor4s    = array[0..3] of smallint;

   {single precision float}
   PColor1f  = ^TColor1f;     TColor1f    = single;
   PColor2f  = ^TColor2f;     TColor2f    = array[0..1] of single;
   PColor3f  = ^TColor3f;     TColor3f    = array[0..2] of single;
   PColor4f  = ^TColor4f;     TColor4f    = array[0..3] of single;

   {double precision float}
   PColor1d  = ^TColor1d;     TColor1d    = double;
   PColor2d  = ^TColor2d;     TColor2d    = array[0..1] of double;
   PColor3d  = ^TColor3d;     TColor3d    = array[0..2] of double;
   PColor4d  = ^TColor4d;     TColor4d    = array[0..3] of double;

   {color tables}
   TColorTable_4_RGB    = array[0..015] of TColor3ub;
   TColorTable_4_RGBA   = array[0..015] of TColor4ub;
   TColorTable_8_RGB    = array[0..255] of TColor3ub;
   TColorTable_8_RGBA   = array[0..255] of TColor4ub;

CONST
   {some frequently used colors}
   cWhite4ub:  TColor4ub = (255, 255, 255, 255);
   cBlack4ub:  TColor4ub = (000, 000, 000, 255);
   cRed4ub:    TColor4ub = (255, 000, 000, 255);
   cGreen4ub:  TColor4ub = (000, 255, 000, 255);
   cBlue4ub:   TColor4ub = (000, 000, 255, 255);

   cWhite4f:   TColor4f  = (1.0, 1.0, 1.0, 1.0);
   cBlack4f:   TColor4f  = (0.0, 0.0, 0.0, 1.0);
   cRed4f:     TColor4f  = (1.0, 0.0, 0.0, 1.0);
   cGreen4f:   TColor4f  = (0.0, 1.0, 0.0, 1.0);
   cBlue4f:    TColor4f  = (0.0, 0.0, 1.0, 1.0);

TYPE
   TColorIDName = string;
   TColorID = record
      ID: longint;
      Name: TColorIDName;
      DClr: PColor4ub;
   end;

VAR
   cWhite3ub:  TColor3ub absolute cWhite4ub;
   cBlack3ub:  TColor3ub absolute cBlack4ub;
   cRed3ub:    TColor3ub absolute cRed4ub;
   cGreen3ub:  TColor3ub absolute cGreen4ub;
   cBlue3ub:   TColor3ub absolute cBlue4ub;

   cWhite3f:   TColor3f absolute cWhite4f;
   cBlack3f:   TColor3f absolute cBlack4f;
   cRed3f:     TColor3f absolute cRed4f;
   cGreen3f:   TColor3f absolute cGreen4f;
   cBlue3f:    TColor3f absolute cBlue4f;

{set colors in a TColor*ub}
procedure clrSet3ub(var clr: TColor4ub; r, g, b: byte); inline;
procedure clrSet4ub(var clr: TColor4ub; r, g, b, a: byte); inline;

procedure clrAssignSingle3f(var clr: TColor3f; c: single);
procedure clrAssignSingle4f(var clr: TColor4f; c: single);
procedure clrAssignSingle3ub(var clr: TColor3ub; c: byte);
procedure clrAssignSingle4ub(var clr: TColor4ub; c: byte);

{ STRING ROUTINES }
function Color4ubToStr(const clr: TColor4ub): string;
function Color3ubToStr(const clr: TColor4ub): string;
function Color4fToStr(const clr: TColor4f; dec: longint): string;
function Color3fToStr(const clr: TColor3f; dec: longint): string;

function HexStrToColor(var s: string; var c: TColor3ub): boolean;
procedure ColorToHexStr(var c: TColor3ub; var s: string);

{ COLOR IDs }
procedure clrGetID(var clrid: TColorID; var idcount: longint);

IMPLEMENTATION

procedure clrSet3ub(var clr: TColor4ub; r, g, b: byte); inline;
begin
   clr[0] := r; clr[1] := g; clr[2] := b;
end;

procedure clrSet4ub(var clr: TColor4ub; r, g, b, a: byte); inline;
begin
   clr[0] := r; clr[1] := g; clr[2] := b; clr[3] := a;
end;

procedure clrAssignSingle3f(var clr: TColor3f; c: single);
begin
   clr[0] := c;
   clr[1] := c;
   clr[2] := c;
end;

procedure clrAssignSingle4f(var clr: TColor4f; c: single);
begin
   clr[0] := c;
   clr[1] := c;
   clr[2] := c;
   clr[3] := c;
end;

procedure clrAssignSingle3ub(var clr: TColor3ub; c: byte);
begin
   clr[0] := c;
   clr[1] := c;
   clr[2] := c;
end;

procedure clrAssignSingle4ub(var clr: TColor4ub; c: byte);
begin
   clr[0] := c;
   clr[1] := c;
   clr[2] := c;
   clr[3] := c;
end;

{ STRING ROUTINES }

function Color4ubToStr(const clr: TColor4ub): string;
begin
   result := dStr(clr[0])+','+dStr(clr[1])+','+dStr(clr[2])+','+dStr(clr[3]);
end;

function Color3ubToStr(const clr: TColor4ub): string;
begin
   result := dStr(clr[0])+','+dStr(clr[1])+','+dStr(clr[2]);
end;

function Color4fToStr(const clr: TColor4f; dec: longint): string;
begin
   result := dStr(clr[0], dec)+','+dStr(clr[1], dec)+','+
      dStr(clr[2], dec)+','+dStr(clr[3], dec);
end;

function Color3fToStr(const clr: TColor3f; dec: longint): string;
begin
   result := dStr(clr[0], dec)+','+dStr(clr[1], dec)+','+dStr(clr[2], dec);
end;

function HexStrToColor(var s: string; var c: TColor3ub): boolean;
var
   l, code: longint;
   clr: TColor3ub;

begin
   HexStrToColor := false;
   l := Length(s);
   if(l = 7) then begin
      if(s[1] = '#') then begin
         val('$'+copy(s, 2, 2), clr[0], code);
         if(code <> 0) then exit;
         val('$'+copy(s, 4, 2), clr[1], code);
         if(code <> 0) then exit;
         val('$'+copy(s, 6, 2), clr[2], code);
         if(code <> 0) then exit;
         c := clr;
         HexStrToColor := true;
      end;
   end;
end;

procedure ColorToHexStr(var c: TColor3ub; var s: string);
begin
   s := '#'+HexStr(c[0], 2)+HexStr(c[1], 2)+HexStr(c[2], 2);
end;

{ COLOR IDs }
procedure clrGetID(var clrid: TColorID; var idcount: longint);
begin
   clrid.ID := idcount;
   inc(idcount);
end;


END.
