{
   uFileStd
   Copyright (C) 2011. Dejan Boras

   This file is part of uFile.

   uFile is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   uFile is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with uFile. If not, see <http://www.gnu.org/licenses/>.

   Started On:    19.02.2011.
   Last Update:   05.04.2011.
}

{$MODE OBJFPC}{$H+}{$I-}
UNIT uFileStd;

INTERFACE

   USES uStd, uError, uFile;
   
VAR
   fStdHandler: TFileHandler;
   fStdHandlerBfrd: TFileHandler;

procedure fOpen(var f: TFile; const fn: string); {normal file}
procedure fNew(var f: TFile; const fn: string); {normal file}   

{copy from a source file to destination}
procedure fCopy(const src: string; var dst: TFile);
procedure fCopy(var src: TFile; const dst: string; size: fileint);

IMPLEMENTATION

{ ... FILE HANDLERS ... }
TYPE
   stdPData = ^stdTData;
   stdTData = record
      f: file;
   end;

{STANDARD FILE HANDLER}
procedure stdMake(var f: TFile);
begin
   new(stdPData(f.pData));
   if(f.pData <> nil) then Zero(f.pData^, SizeOf(stdTData))
   else feRaise(eNO_MEMORY);
end;

procedure stdDispose(var f: TFile);
begin
   if(f.pData <> nil) then begin
      dispose(stdPData(f.pData)); f.pData := nil;
   end;
end;

procedure stdOpen(var f: TFile);
var
   data: stdPData absolute f.pData;

begin
   if(data <> nil) then begin
      Reset(data^.f, 1);
      if(ioerror() <> 0) then feRaise(feOPEN)
   end;
end;

procedure stdNew(var f: TFile);
var
   data: stdPData absolute f.pData;

begin
   if(data <> nil) then begin
      Rewrite(data^.f, 1);
      if(ioerror() <> 0) then feRaise(feNEW);
   end;
end;

procedure stdClose(var f: TFile);
var
   data: stdPData absolute f.pData;

begin
   if(data <> nil) then begin
      Close(data^.f);
      if(ioerror() <> 0) then feRaise(eIO);
   end;
end;

procedure stdFlush(var f: TFile);
var
   data: stdPData absolute f.pData;

begin
   blockwrite(data^.f, f.bData^, f.bPosition);
   if(ioerror() <> 0) then feRaise(eIO);
end;

function stdReadBfrd(var f: TFile; var buf; count: fileint): fileint;
var
   data: stdPData absolute f.pData;
   bRead, bLeft, rLeft: fileint;

begin
   bLeft := f.bLimit-f.bPosition;

   {if enough data is in the buffer}
   if(count <= bLeft) then begin
      move((f.bData+f.bPosition)^, buf, count);
      inc(f.bPosition, count);

      result := count;
   end else begin
      {move what can be moved from the buffer}
      if(bLeft > 0) then begin
         move((f.bData+f.bPosition)^, buf, bLeft);
         f.bPosition := 0;
         f.bLimit := 0;
      end;

      {figure out what is left}
      rLeft := (count-bLeft);

      {fill the buffer first and then read from buffer}
      if(rLeft <= f.bSize) then begin
         blockread(data^.f, f.bData^, f.bSize, bRead);
         if(ioerror() = 0) then begin
            move(f.bData^, (@buf+bLeft)^, rLeft);
            f.bPosition := rLeft;
            f.bLimit := bRead;
            result := count;
         end else begin
            feRaise(eIO); result := -bLeft;
         end;
      {read directly}
      end else begin
         blockread(data^.f, (@buf+bLeft)^, rLeft, bRead);
         if(ioerror() = 0) then
            result := bLeft+bRead
         else begin
            feRaise(eIO); result := -(bLeft+bRead);
         end;
      end;
   end;
end;

function stdRead(var f: TFile; var buf; count: fileint): fileint;
var
   data: stdPData absolute f.pData;
   bRead: fileint;

begin
   blockread(data^.f, buf, count, bRead);

   if(ioerror() = 0) then
      result := bRead
   else begin feRaise(eIO); exit(-bRead); end;
end;

function stdWriteBfrd(var f: TFile; var buf; count: fileint): fileint;
var
   data: stdPData absolute f.pData;
   bWrite: fileint;

procedure movetobuf(); inline;
begin
   move(buf, (f.bData+f.bPosition)^, count);
   inc(f.bPosition, count);
end;

begin
   {store into buffer}
   if(f.bPosition+count < f.bSize-1) then begin
      movetobuf();
      result := count;
   {write out}
   end else begin
      {write out buffer}
      blockwrite(data^.f, f.bData^, f.bPosition, bWrite);
      f.bPosition := 0;

      {store data into buffer if it can fit}
      if(count < f.bSize) then begin
         movetobuf();
         result := count;
      {otherwise write contents directly to file}
      end else begin
         blockwrite(data^.f, buf, count, bWrite);

         if(ioerror() = 0) then
            result := bWrite
         else begin feRaise(eIO); exit(-bWrite); end
      end;
   end;
end;

function stdWrite(var f: TFile; var buf; count: fileint): fileint;
var
   data: stdPData absolute f.pData;
   bWrite: fileint;

begin
   blockwrite(data^.f, buf, count, bWrite);
   if(ioerror() = 0) then
      result := bWrite
   else begin feRaise(eIO); exit(-bWrite); end
end;

procedure stdSeek(var f: TFile; pos: fileint);
var
   data: stdPData absolute f.pData;

begin
   if(data <> nil) then begin
      seek(data^.f, pos);
      if(ioerror() <> 0) then feRaise(eIO);
   end;
end;

procedure stdOnBufferSet(var f: TFile);
begin
   {if buffering set}
   if(f.bSize > 0) then
      f.pHandler := @fStdHandlerBfrd
   {if buffering not set}
   else
      f.pHandler := @fStdHandler;
end;

procedure fOpen(var f: TFile; const fn: string);
var
   data: stdPData absolute f.pData;

begin
   fErrorReset();

   {$IFNDEF FILE_NOFS}
   if(fsExists(fn) > -1) then begin
      fsOpen(f, fn); exit;
   end;
   {$ENDIF}

   f.fNew := 0;
   f.fMode := fcfREAD;

   {set defaults}
   fSetDefaults(f, fn);

   {assign a standard file handler}
   fAssignHandler(f, fStdHandler);
   if(fError = 0) then begin
      if(data <> nil) then begin
         Assign(data^.f, fn);

         stdOpen(f);
         if(fError = 0) then begin
            f.fSize := FileSize(data^.f);
            f.fSizeLimit := 0;
         end else
            fDispose(f);
      end;
   end;
end;

procedure fNew(var f: TFile; const fn: string);
var
   data: stdPData absolute f.pData;

begin
   fErrorReset();
   f.fNew := 1;
   f.fMode := fcfWRITE;

   {set defaults}
   fSetDefaults(f, fn);

   {assign a standard file handler}
   fAssignHandler(f, fStdHandler);
   if(fError = 0) then begin
      if(data <> nil) then begin
         Assign(data^.f, fn);

         stdNew(f);
      end;
   end;
end;

function getCopyBuffer(): pbyte;
var
   buf: pbyte = nil;

begin
   GetMem(buf, fcCopyBufferSize);
   if(buf = nil) then feRaise(eNO_MEMORY);
   result := buf;
end;

procedure fCopy(const src: string; var dst: TFile);
var
   buf: pbyte;
   f: file;
   bsize, bread: longint;

   procedure cleanup();
   begin
      XFreeMem(buf);
   end;

begin
   buf := getCopyBuffer();
   if(buf <> nil) then begin
      {open source file}
      Assign(f, src);
      Reset(f, 1);
      if(ioerror() = 0) then begin

         bsize := fcCopyBufferSize;
         {copy}
         repeat
            blockread(f, buf^, bsize, bread);
            if(ioerror() = 0) then begin
               if(bread > 0) then
                  fWrite(dst, buf^, bread)
               else
                  break;
            end else begin
               feRaise(eIO); break;
            end;
         until eof(f);

         cleanup(); Close(f); IOResult();
      end else begin
         cleanup(); feRaise(eIO);
      end;
   end;
end;

procedure fCopy(var src: TFile; const dst: string; size: fileint);
var
   f: file;
   buf: pbyte;
   bsize: longint;
   siz, rd: longint;

   procedure cleanup();
   begin
      XFreeMem(buf);
      Close(f); IOResult();
   end;

begin
   buf := getCopyBuffer();

   if(fError = 0) then begin
      {create file}
      assign(f, dst);
      rewrite(f, 1);
      if(ioerror() = 0) then begin
         siz := size;
         bsize := fcCopyBufferSize;

         {copy}
         repeat
            rd := bsize;
            if(rd > siz) then rd := siz;

            fRead(src, buf^, rd);
            if(fError = 0) then begin
               blockwrite(f, buf^, rd);
               if(ioerror() <> 0) then begin
                  feRaise(eIO); cleanup(); exit;
               end;

               dec(siz, rd);
            end else break;
         until (siz <= 0);
      end else
         feRaise(eIO);

      {done}
      cleanup();
   end;
end;

INITIALIZATION
   {standard file handler}
   fStdHandler := fDummyHandler;
   fStdHandler.make       := fTMakeProc     (@stdMake);
   fStdHandler.dispose    := fTMakeProc     (@stdDispose);
   fStdHandler.open       := fTOpenProc     (@stdOpen);
   fStdHandler.read       := fTReadFunc     (@stdRead);
   fStdHandler.write      := fTWriteFunc    (@stdWrite);
   fStdHandler.close      := fTCloseProc    (@stdClose);
   fStdHandler.flush      := fTFlushProc    (@stdFlush);
   fStdHandler.seek       := fTSeekProc     (@stdSeek);
   fStdHandler.onbufferset:= ftOnBufferSetProc(@stdOnBufferSet);
   fStdHandler.useBuffering := true;

   fStdHandlerBfrd := fStdHandler;
   fStdHandlerBfrd.read := fTReadFunc(@stdReadBfrd);
   fStdHandlerBfrd.write := fTWriteFunc(@stdWriteBfrd);
END.
