{
   frmuModSelect, mod selection form
   Copyright (c) 2011. Dejan Boras

   This file is part of OpenArena Name Changer (oanc).

   OpenArena Name Changer (oanc) is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
   (at your option) any later version.

   OpenArena Name Changer (oanc) is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with OpenArena Name Changer (oanc). If not, see <http://www.gnu.org/licenses/>.

   Started On:    07.03.2011.
   Last Update:   22.08.2011.
}

{$MODE OBJFPC}{$H+}
UNIT frmuModSelect;

INTERFACE

USES
  Classes, SysUtils, FileUtil, LResources, Forms, Controls, Graphics, Dialogs,
  Buttons, CheckLst, StdCtrls, ExtCtrls,
  {oanc}
  uoanc, frmuPreview;

TYPE

  { TfrmModSelect }

  TfrmModSelect = class(TForm)
     btnAction: TBitBtn;
     btnCancel: TBitBtn;
     btnSelectAllNone: TBitBtn;
     btnSingleItemAction: TButton;
     cblMods: TCheckListBox;
     edSource: TEdit;
     lblChoose: TLabel;
     lblCurName: TLabel;
     lblNameSource: TLabel;
     shpPrv: TShape;
     procedure btnCancelClick(Sender: TObject);
     procedure btnActionClick(Sender: TObject);
     procedure btnSelectAllNoneClick(Sender: TObject);
     procedure btnSingleItemActionClick(Sender: TObject);
     procedure cblModsClick(Sender: TObject);
     procedure FormResize(Sender: TObject);
     procedure FormShow(Sender: TObject);
     procedure shpPrvMouseUp(Sender: TObject; Button: TMouseButton;
        Shift: TShiftState; X, Y: Integer);
     procedure shpPrvPaint(Sender: TObject);
  private
    { private declarations }
  public
    { public declarations }
  end; 

TYPE
   TSelectActionFunc = function(): boolean;

VAR
  frmModSelect: TfrmModSelect;

  {a callback for the Action button}
  SelectActionFunc: TSelectActionFunc = nil;
  SingleItemActionFunc: TSelectActionFunc = nil;

  {list of names from mods, this is required so the mods cfg files are not read every time}
  namesList: TStringList;

{sets the preview to the specified string}
procedure SetPreview(const selName: string);

IMPLEMENTATION

{ TfrmModSelect }

VAR
   {used to toggle the select/unsellect all button}
   selectstate: boolean = false;
   {are there any mods?}
   hasMods: boolean = false;
   {the player name for the selected mod}
   SelectedName: string = '';

procedure select(s: boolean);
var
   clb: TCheckListBox;
   i: longint;

begin
   clb := frmModSelect.cblMods;

   if(clb.Items.Count > 0) then begin
      for i := 0 to (clb.Items.Count-1) do
         clb.Checked[i] := s;
   end;
end;

procedure SetSelectStateButtonCaptions();
begin
   case selectstate of
      true:  frmModSelect.btnSelectAllNone.Caption := 'Select None';
      false: frmModSelect.btnSelectAllNone.Caption := 'Select All';
   end;
end;

procedure SetSelectState();
begin
   case selectstate of
      true:  select(true);
      false: select(false);
   end;

   SetSelectStateButtonCaptions();
end;

procedure TfrmModSelect.btnSelectAllNoneClick(Sender: TObject);
begin
   selectstate := selectstate xor true;

   SetSelectState();
end;

procedure TfrmModSelect.btnSingleItemActionClick(Sender: TObject);
begin
   SingleItemActionFunc();
end;

procedure SetPreview(const selName: string);
begin
   SelectedName := selName;
   frmModSelect.shpPrv.Invalidate();
end;

procedure TfrmModSelect.cblModsClick(Sender: TObject);
var
   i: longint;
   modsChecked: longint;

begin
   SetPreview('');

   {if a valid item was selected}
   if(cblMods.ItemIndex >= 0) then begin
      if(cblMods.ItemIndex < namesList.Count) then begin
         SelectedName := namesList.Strings[cblMods.ItemIndex];
         edSource.Text := namesList.Strings[cblMods.ItemIndex];
      end;
      {show the player name from the selected mod file}
   end;

   {check if all mods are selected}
   modsChecked := 0;

   for i := 0 to (cblMods.Count-1) do begin
      if(cblMods.Checked[i] = false) then
         inc(modsChecked);
   end;

   if(modsChecked = 0) then begin
      selectstate := true;
      SetSelectStateButtonCaptions();
   end else begin
      selectstate := false;
      SetSelectStateButtonCaptions();
   end;
end;

VAR
   prevClientHeight: longint = 0;

procedure TfrmModSelect.FormResize(Sender: TObject);
var
   diff: longint;

begin
   {horizontal resize}
   cblMods.Width := frmModSelect.ClientWidth - cblMods.Left*2;
   shpPrv.Width := frmModSelect.ClientWidth - shpPrv.Left*2;
   edSource.Width := frmModSelect.ClientWidth - (edSource.Left+lblNameSource.Left);

   {vertical resize}
   if(prevClientHeight = 0) then begin
      prevClientHeight := frmModSelect.ClientHeight;
      diff := frmModSelect.ClientHeight - frmModSelect.Constraints.MinHeight;
   end else
      diff := frmModSelect.ClientHeight - prevClientHeight;

   prevClientHeight := frmModSelect.ClientHeight;

   if(diff <> 0) then begin
      cblMods.Height := cblMods.Height + diff;
      lblNameSource.Top := lblNameSource.Top + diff;
      lblCurName.Top := lblCurName.Top + diff;
      shpPrv.Top := shpPrv.Top + diff;
      edSource.Top := edSource.Top + diff;
      btnAction.Top := btnAction.Top + diff;
      btnCancel.Top := btnCancel.Top + diff;
      btnSelectAllNone.Top := btnSelectAllNone.Top + diff;
      btnSingleItemAction.Top := btnSingleItemAction.Top + diff;
   end;
end;

procedure TfrmModSelect.btnCancelClick(Sender: TObject);
begin
   Close();
end;

procedure TfrmModSelect.btnActionClick(Sender: TObject);
begin
   if(SelectActionFunc <> nil) then begin
      if(SelectActionFunc()) then Close();
   {this should never be called, as a SelectActionFunc must always be set}
   end else Close();
end;

procedure TfrmModSelect.FormShow(Sender: TObject);
begin
   {when the form is opened, no mod is selected initially}
   SelectedName := '';
   edSource.Text := '';

   {check if there are any mods}
   hasMods := oaHasMods();

   {add mods to the list}
   if(hasMods) then begin
      cblMods.Items.Clear();
      cblMods.Items.Assign(modlist);
      btnSelectAllNone.Enabled := true;
   end else begin
      cblMods.Items.Clear();
      cblMods.Items.Add('Selected config file');
      btnSelectAllNone.Enabled := false;
   end;

   {set state}
   selectstate := false;
   SetSelectState();
end;

procedure TfrmModSelect.shpPrvMouseUp(Sender: TObject; Button: TMouseButton;
   Shift: TShiftState; X, Y: Integer);
begin
   if(Button = mbLeft) then begin
      sPreviewName := SelectedName;
      frmPreview.ShowModal();
   end;
end;

{ RENDER PREVIEW }

procedure TfrmModSelect.shpPrvPaint(Sender: TObject);
begin
   PreviewName(SelectedName, shpPrv);
end;

INITIALIZATION
  {$I frmuModSelect.lrs}
  namesList := TStringList.Create();

END.

